'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _utils = require('./utils.js');

var _RouteOptions = require('./RouteOptions.js');

var _RouteOptions2 = _interopRequireDefault(_RouteOptions);

var _RouteParams = require('./RouteParams.js');

var _RouteParams2 = _interopRequireDefault(_RouteParams);

var _TemplateUri = require('./TemplateUri.js');

var _TemplateUri2 = _interopRequireDefault(_TemplateUri);

var _UserUri = require('./UserUri.js');

var _UserUri2 = _interopRequireDefault(_UserUri);

var _RouterError = require('./RouterError.js');

var _RouterError2 = _interopRequireDefault(_RouterError);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var metaDataProperties = ['payload', 'key', 'data'];

var Route = function () {
    function Route(routeName, rawRoute) {
        _classCallCheck(this, Route);

        var rawRouteParams = rawRoute.params || {};
        var rawRouteOptions = rawRoute.options || {};
        var rawRouteUri = rawRoute.uri || '';

        if ((typeof rawRouteParams === 'undefined' ? 'undefined' : _typeof(rawRouteParams)) !== 'object') {
            throw new _RouterError2.default(_RouterError2.default.INVALID_INPUT_TYPE, {
                entity: 'route\'s params',
                type: 'object',
                routeName: routeName
            });
        }
        if ((typeof rawRouteOptions === 'undefined' ? 'undefined' : _typeof(rawRouteOptions)) !== 'object') {
            throw new _RouterError2.default(_RouterError2.default.INVALID_INPUT_TYPE, {
                entity: 'route\'s options',
                type: 'object',
                routeName: routeName
            });
        }
        if (typeof rawRouteUri !== 'string') {
            throw new _RouterError2.default(_RouterError2.default.INVALID_INPUT_TYPE, {
                entity: 'route\'s URI template',
                type: 'string',
                routeName: routeName
            });
        }

        this.name = routeName;
        var templateUri = new _TemplateUri2.default(rawRouteUri);
        var parsedOptions = new _RouteOptions2.default(rawRouteOptions, routeName);
        var parsedParams = new _RouteParams2.default(rawRouteParams);
        var parsedTemplate = {};
        (0, _utils.forEachPartName)(function (partName) {
            var Template = (0, _utils.chooseTemplate)(templateUri, partName);
            parsedTemplate[partName] = new Template(templateUri, parsedParams, routeName);
        });
        var parsedMetaData = {};
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = metaDataProperties[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var metaDataProperty = _step.value;

                parsedMetaData[metaDataProperty] = rawRoute[metaDataProperty];
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }

        this._templateUri = templateUri;
        this._parsedOptions = parsedOptions;
        this._parsedParams = parsedParams;
        this._parsedTemplate = parsedTemplate;
        this._parsedMetaData = parsedMetaData;
    }

    _createClass(Route, [{
        key: 'matchUri',
        value: function matchUri(userUri, contextOptions) {
            var routeName = this.name;
            var parsedTemplate = this._parsedTemplate;
            var matchObject = {};
            var params = {};
            var options = Object.assign({}, contextOptions.router.getParsedOptions(), this._parsedOptions);
            Object.assign(contextOptions, options);

            for (var p in parsedTemplate) {
                contextOptions.partName = p;
                var matchFragment = parsedTemplate[p].matchParsedValue(userUri, contextOptions);
                if (!matchFragment) {
                    return null;
                }
                matchObject[p] = matchFragment.value;
                Object.assign(params, matchFragment.params);
            }

            delete options.routeName;
            Object.assign(matchObject, this._parsedMetaData, {
                name: routeName,
                params: params,
                options: options
            });
            return matchObject;
        }
    }, {
        key: 'generateUri',
        value: function generateUri(userParams, contextOptions) {
            var routeName = this.name;
            var parsedTemplate = this._parsedTemplate;
            var generatingUri = {};
            Object.assign(contextOptions, contextOptions.router.getParsedOptions(), this._parsedOptions, { generatingUri: generatingUri, routeName: routeName });
            for (var p in parsedTemplate) {
                contextOptions.partName = p;
                generatingUri[p] = parsedTemplate[p].generateParsedValue(userParams, contextOptions);
            }

            var rawUri = void 0;
            try {
                rawUri = (0, _utils.joinUri)(generatingUri, contextOptions);
            } catch (error) {
                throw new _RouterError2.default(_RouterError2.default[error.code], { routeName: routeName });
            }

            if (contextOptions.dataConsistency) {
                var userUri = new _UserUri2.default(contextOptions.router.resolveUri(rawUri), contextOptions);
                if (!this.matchUri(userUri, { router: contextOptions.router })) {
                    throw new _RouterError2.default(_RouterError2.default.INCONSISTENT_DATA, { routeName: routeName });
                }
            }

            return rawUri;
        }
    }, {
        key: 'canBeMatched',
        value: function canBeMatched() {
            var parsedOptions = this._parsedOptions;
            return parsedOptions.canBeMatched;
        }
    }, {
        key: 'canBeGenerated',
        value: function canBeGenerated() {
            var parsedOptions = this._parsedOptions;
            return parsedOptions.canBeGenerated;
        }
    }, {
        key: 'getParsedOptions',
        value: function getParsedOptions() {
            return this._parsedOptions;
        }
    }]);

    return Route;
}();

exports.default = Route;