'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _utils = require('./utils.js');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var RouterError = function (_Error) {
    _inherits(RouterError, _Error);

    function RouterError(errorTemplate) {
        var errorOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

        _classCallCheck(this, RouterError);

        var message = errorTemplate.code + ' ★ ';
        message += errorTemplate(errorOptions);
        var routeName = errorOptions.routeName;

        if (routeName) {
            message += ' Check the route ' + routeName + '.';
        }

        return _possibleConstructorReturn(this, (RouterError.__proto__ || Object.getPrototypeOf(RouterError)).call(this, message));
    }

    return RouterError;
}(Error);

exports.default = RouterError;


var errorTemplates = {
    INVALID_ROUTE_NAME: function INVALID_ROUTE_NAME(_ref) {
        var desiredRouteName = _ref.desiredRouteName;
        return 'Invalid route\'s name; ' + desiredRouteName + ' was not found!';
    },
    INVALID_INPUT_TYPE: function INVALID_INPUT_TYPE(_ref2) {
        var entity = _ref2.entity,
            type = _ref2.type;
        return (0, _utils.toTitleCase)(entity) + ' must be presented by ' + (0, _utils.toLowerCase)(type) + '.';
    },
    INVALID_URI_STRUCTURE: function INVALID_URI_STRUCTURE(_ref3) {
        var entity = _ref3.entity;
        return (0, _utils.toTitleCase)(entity) + ' cannot be splitted because it has invalid structure.';
    },
    INVALID_PATH_STRUCTURE: function INVALID_PATH_STRUCTURE(_ref4) {
        var entity = _ref4.entity;
        return 'Path in ' + (0, _utils.toLowerCase)(entity) + ' must be absolute when domain is specified!';
    },
    INVALID_DEFAULT_GETTER: function INVALID_DEFAULT_GETTER() {
        return 'Method getDefaultPart must be presented by function that returns instances of default URI components.';
    },
    INCONSISTENT_DATA: function INCONSISTENT_DATA() {
        return 'Data is inconsistent; generated URI cannot be matched.';
    },
    MATCH_FRAGMENT_EXPECTED: function MATCH_FRAGMENT_EXPECTED() {
        return 'Match function must return null or MatchFragment\'s instance.';
    },
    FUNCTION_EXPECTED: function FUNCTION_EXPECTED(_ref5) {
        var entity = _ref5.entity;
        return 'Only function could be used as parameter for the whole ' + (0, _utils.toLowerCase)(entity) + '.';
    },
    DOMAIN_EXPECTED: function DOMAIN_EXPECTED() {
        return 'URI must contain not empty domain when protocol or port is specified.';
    },
    ABSOLUTE_PATH_EXPECTED: function ABSOLUTE_PATH_EXPECTED(_ref6) {
        var currentPath = _ref6.currentPath;
        return 'Matching URI path must be absolute. Current path is ' + currentPath + '.';
    },
    PATH_COMPONENT_EXPECTED: function PATH_COMPONENT_EXPECTED(_ref7) {
        var paramName = _ref7.paramName;
        return 'Path component cannot be empty.' + (paramName ? ' Look at the parameter ' + paramName + '.' : '');
    },
    CUSTOM_ERROR: function CUSTOM_ERROR(_ref8) {
        var message = _ref8.message;
        return '' + message;
    }
};
for (var e in errorTemplates) {
    errorTemplates[e].code = e;
}
Object.assign(RouterError, errorTemplates);