'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.convertGenerateItemsToFunctions = exports.getFallbackGenerateFunction = exports.getParamGenerateFunction = exports.getGenerateFunctionByItem = exports.convertMatchItemsToFunctions = exports.getFallbackMatchFunction = exports.getMatchFunctionByItem = exports.joinUri = exports.splitUri = exports.parseSplittedUriPart = exports.emulateParsedUri = exports.chooseTemplate = exports.getPortByParsedUri = exports.getRegExp = exports.getPortByProtocol = exports.getDefaultPart = exports.getPartConstructor = exports.forEachPartName = exports.isMatchFragment = exports.isMatchGenerate = exports.isEmpty = exports.toTitleCase = exports.toUpperCase = exports.toLowerCase = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _Protocol = require('./Protocol.js');

var _Protocol2 = _interopRequireDefault(_Protocol);

var _Domain = require('./Domain.js');

var _Domain2 = _interopRequireDefault(_Domain);

var _Port = require('./Port.js');

var _Port2 = _interopRequireDefault(_Port);

var _Path = require('./Path.js');

var _Path2 = _interopRequireDefault(_Path);

var _Query = require('./Query.js');

var _Query2 = _interopRequireDefault(_Query);

var _Hash = require('./Hash.js');

var _Hash2 = _interopRequireDefault(_Hash);

var _PathComponent = require('./PathComponent.js');

var _PathComponent2 = _interopRequireDefault(_PathComponent);

var _QueryComponent = require('./QueryComponent.js');

var _QueryComponent2 = _interopRequireDefault(_QueryComponent);

var _MatchFragment = require('./MatchFragment.js');

var _MatchFragment2 = _interopRequireDefault(_MatchFragment);

var _RouterError = require('./RouterError.js');

var _RouterError2 = _interopRequireDefault(_RouterError);

var _ParamProtocolTemplate = require('./ParamProtocolTemplate.js');

var _ParamProtocolTemplate2 = _interopRequireDefault(_ParamProtocolTemplate);

var _ParamDomainTemplate = require('./ParamDomainTemplate.js');

var _ParamDomainTemplate2 = _interopRequireDefault(_ParamDomainTemplate);

var _ParamPortTemplate = require('./ParamPortTemplate.js');

var _ParamPortTemplate2 = _interopRequireDefault(_ParamPortTemplate);

var _ParamPathTemplate = require('./ParamPathTemplate.js');

var _ParamPathTemplate2 = _interopRequireDefault(_ParamPathTemplate);

var _ParamQueryTemplate = require('./ParamQueryTemplate.js');

var _ParamQueryTemplate2 = _interopRequireDefault(_ParamQueryTemplate);

var _ParamHashTemplate = require('./ParamHashTemplate.js');

var _ParamHashTemplate2 = _interopRequireDefault(_ParamHashTemplate);

var _ConstProtocolTemplate = require('./ConstProtocolTemplate.js');

var _ConstProtocolTemplate2 = _interopRequireDefault(_ConstProtocolTemplate);

var _ConstDomainTemplate = require('./ConstDomainTemplate.js');

var _ConstDomainTemplate2 = _interopRequireDefault(_ConstDomainTemplate);

var _ConstPortTemplate = require('./ConstPortTemplate.js');

var _ConstPortTemplate2 = _interopRequireDefault(_ConstPortTemplate);

var _ConstPathTemplate = require('./ConstPathTemplate.js');

var _ConstPathTemplate2 = _interopRequireDefault(_ConstPathTemplate);

var _ConstQueryTemplate = require('./ConstQueryTemplate.js');

var _ConstQueryTemplate2 = _interopRequireDefault(_ConstQueryTemplate);

var _ConstHashTemplate = require('./ConstHashTemplate.js');

var _ConstHashTemplate2 = _interopRequireDefault(_ConstHashTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var partTemplates = {
    ParamProtocolTemplate: _ParamProtocolTemplate2.default,
    ParamDomainTemplate: _ParamDomainTemplate2.default,
    ParamPortTemplate: _ParamPortTemplate2.default,
    ParamPathTemplate: _ParamPathTemplate2.default,
    ParamQueryTemplate: _ParamQueryTemplate2.default,
    ParamHashTemplate: _ParamHashTemplate2.default,
    ConstProtocolTemplate: _ConstProtocolTemplate2.default,
    ConstDomainTemplate: _ConstDomainTemplate2.default,
    ConstPortTemplate: _ConstPortTemplate2.default,
    ConstPathTemplate: _ConstPathTemplate2.default,
    ConstQueryTemplate: _ConstQueryTemplate2.default,
    ConstHashTemplate: _ConstHashTemplate2.default
};

var partNames = ['protocol', 'domain', 'port', 'path', 'query', 'hash'];

var partConstructors = {
    protocol: _Protocol2.default,
    domain: _Domain2.default,
    port: _Port2.default,
    path: _Path2.default,
    query: _Query2.default,
    hash: _Hash2.default,
    queryComponent: _QueryComponent2.default,
    pathComponent: _PathComponent2.default
};

var portByProtocol = {
    ftp: 21,
    ssh: 22,
    telnet: 23,
    smtp: 25,
    dns: 53,
    dhcp: 68,
    tftp: 69,
    http: 80,
    pop: 110,
    ntp: 123,
    imap: 143,
    snmp: 162,
    bgp: 179,
    https: 443,
    ldaps: 636
};

var regExps = {
    uri: /^(?:([+A-z0-9]+:)?\/\/([-.A-zА-я0-9.]+)(:[0-9]+)?)?([^?#]*)(\?[^#]*)?(#.*)?$/i,
    template: /^(?:((?:[+A-z0-9]+|{[A-z0-9]+}):)?\/\/((?:[-.A-zА-я0-9]+|{[A-z0-9]+}))(:(?:[0-9]+|{[A-z0-9]+}))?)?([^?#]*)(\?[^#]*)?(#.*)?$/i,
    param: /^{([^*}]*)(\*)?}$/
};

var splittedUriPartRegExps = {
    protocol: /:$/,
    domain: null,
    port: /^:/,
    path: null,
    query: /^\?/,
    hash: /^#/
};

var toLowerCase = exports.toLowerCase = function toLowerCase() {
    var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    return value.toLowerCase();
};

var toUpperCase = exports.toUpperCase = function toUpperCase() {
    var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    return value.toUpperCase();
};

var toTitleCase = exports.toTitleCase = function toTitleCase() {
    var value = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

    return value.substr(0, 1).toUpperCase() + value.substr(1);
};

var isEmpty = exports.isEmpty = function isEmpty(value) {
    return value === null || value === undefined;
};

var isMatchGenerate = exports.isMatchGenerate = function isMatchGenerate(value) {
    return value && (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object' && (value.match || value.generate);
};

var isMatchFragment = exports.isMatchFragment = function isMatchFragment(value) {
    if (value === null) {
        return true;
    }
    return value && value.hasOwnProperty('value') && value.hasOwnProperty('params');
};

var forEachPartName = exports.forEachPartName = function forEachPartName(callback) {
    partNames.forEach(callback);
};

var getPartConstructor = exports.getPartConstructor = function getPartConstructor(partName) {
    return partConstructors[partName];
};

var getDefaultPart = exports.getDefaultPart = function getDefaultPart(partName) {
    var PartConstructor = getPartConstructor(partName);
    return new PartConstructor();
};

var getPortByProtocol = exports.getPortByProtocol = function getPortByProtocol(protocol) {
    protocol = (protocol + '').toLowerCase();
    return new _Port2.default(portByProtocol[protocol] || null);
};

var getRegExp = exports.getRegExp = function getRegExp(regExpName) {
    return regExps[regExpName];
};

var getPortByParsedUri = exports.getPortByParsedUri = function getPortByParsedUri(_ref, _ref2) {
    var domain = _ref.domain,
        protocol = _ref.protocol,
        port = _ref.port;
    var router = _ref2.router;

    if (port.isEmpty()) {
        if (domain.isEmpty()) {
            var defaultPort = router.getDefaultPart('port');
            if (defaultPort.isEmpty()) {
                var defaultProtocol = router.getDefaultPart('protocol');
                var portByDefaultProtocol = getPortByProtocol(defaultProtocol.toLowerCase(true).toString());
                if (portByDefaultProtocol.isEmpty()) {
                    return defaultPort;
                } else {
                    return portByDefaultProtocol;
                }
            } else {
                return defaultPort;
            }
        } else {
            var currentProtocol = protocol.isEmpty() ? router.getDefaultPart('protocol') : protocol;
            var portByCurrentProtocol = getPortByProtocol(currentProtocol.toLowerCase(true).toString());
            if (portByCurrentProtocol.isEmpty()) {
                return port;
            } else {
                return portByCurrentProtocol;
            }
        }
    } else {
        return port;
    }
};

var chooseTemplate = exports.chooseTemplate = function chooseTemplate(templateUri, partName) {
    var templateType = void 0;
    var rawTemplate = templateUri.getSplittedUri(partName);

    if (rawTemplate) {
        var paramTest = getRegExp('param').test(rawTemplate);
        templateType = paramTest ? 'Param' : 'Const';
    } else {
        templateType = 'Const';
    }

    var templateName = templateType + toTitleCase(partName) + 'Template';
    return partTemplates[templateName];
};

var emulateParsedUri = exports.emulateParsedUri = function emulateParsedUri(templateUri, parsedUri) {
    var emulatedParsedUri = {
        port: templateUri.getParsedUri('port')
    };
    var templateUriDomain = templateUri.getParsedUri('domain');
    if (templateUriDomain.isEmpty()) {
        emulatedParsedUri.protocol = templateUri.getParsedUri('protocol');
        emulatedParsedUri.domain = templateUriDomain;
    } else {
        emulatedParsedUri.protocol = parsedUri.protocol;
        emulatedParsedUri.domain = parsedUri.domain;
    }
    return emulatedParsedUri;
};

var parseSplittedUriPart = exports.parseSplittedUriPart = function parseSplittedUriPart(rawUriPart, regExp) {
    if (!rawUriPart) {
        return null;
    }
    if (regExp) {
        rawUriPart = rawUriPart.replace(regExp, '');
    }
    return rawUriPart || null;
};

var splitUri = exports.splitUri = function splitUri(rawUri, regExp) {
    var uriMatch = rawUri.match(regExp);

    if (!uriMatch) {
        var uriStructureError = new Error('URI has invalid structure and cannot be splitted!');
        uriStructureError.code = 'INVALID_URI_STRUCTURE';
        throw uriStructureError;
    }
    if (uriMatch[2] && uriMatch[4][0] !== '/' && !getRegExp('param').test(uriMatch[4])) {
        if (uriMatch[4]) {
            var pathStructureError = new Error('Path must be absolute when domain is specified!');
            pathStructureError.code = 'INVALID_PATH_STRUCTURE';
            throw pathStructureError;
        } else {
            uriMatch[4] = '/';
        }
    }

    var splittedUri = {};
    forEachPartName(function (partName, p) {
        var regExp = splittedUriPartRegExps[partName];
        splittedUri[partName] = parseSplittedUriPart(uriMatch[p + 1], regExp);
    });
    return splittedUri;
};

var joinUri = exports.joinUri = function joinUri(parsedUri, _ref3) {
    var router = _ref3.router;
    var protocol = parsedUri.protocol,
        domain = parsedUri.domain,
        port = parsedUri.port,
        path = parsedUri.path,
        query = parsedUri.query,
        hash = parsedUri.hash;

    var rawUri = '';

    var defaultProtocol = router.getDefaultPart('protocol');
    var canProtocolBeOmitted = protocol.isEmpty() || _Protocol2.default.isEqual(protocol, defaultProtocol);
    if (!canProtocolBeOmitted) {
        rawUri += protocol + ':';
    }

    if (protocol.isEmpty()) {
        protocol = defaultProtocol;
    }
    var defaultDomain = router.getDefaultPart('domain');
    var canDomainBeOmitted = domain.isEmpty() || _Domain2.default.isEqual(domain, defaultDomain);
    var portByProtocol = getPortByProtocol(protocol.toLowerCase(true).toString());
    var defaultPort = router.getDefaultPart('port');
    var canPortBeOmitted = false || port.isEmpty() || _Port2.default.isEqual(port, portByProtocol) || canProtocolBeOmitted && canDomainBeOmitted && _Port2.default.isEqual(port, defaultPort);
    if (!canProtocolBeOmitted || !canDomainBeOmitted || !canPortBeOmitted) {
        if (domain.isEmpty()) {
            var domainExpectedError = new Error('Domain must be not empty when protocol or port are specified.');
            domainExpectedError.code = 'DOMAIN_EXPECTED';
            throw domainExpectedError;
        } else {
            rawUri += '//' + domain;
        }
        if (!canPortBeOmitted) {
            rawUri += ':' + port;
        }
    }

    if (!domain.isEmpty() && !path.isAbsolute()) {
        var pathStructureError = new Error('Path must be absolute when domain is specified!');
        pathStructureError.code = 'INVALID_PATH_STRUCTURE';
        throw pathStructureError;
    }
    rawUri += path;

    if (!query.isEmpty()) {
        rawUri += '?' + query;
    }

    if (!hash.isEmpty()) {
        rawUri += '#' + hash;
    }

    return rawUri;
};

var getMatchFunctionByItem = exports.getMatchFunctionByItem = function getMatchFunctionByItem(matchItem, _ref4) {
    var partName = _ref4.partName,
        paramName = _ref4.paramName,
        getUserUriPart = _ref4.getUserUriPart;

    if (typeof matchItem === 'function') {
        return function (userUri, contextOptions) {
            contextOptions.paramName = paramName;
            contextOptions.MatchFragment = _MatchFragment2.default;
            var matchFragment = matchItem(userUri, contextOptions);
            if (!isMatchFragment(matchFragment)) {
                throw new _RouterError2.default(_RouterError2.default.MATCH_FRAGMENT_EXPECTED);
            }
            return matchFragment;
        };
    } else if (matchItem instanceof RegExp) {
        return function (userUri, contextOptions) {
            var userUriPart = getUserUriPart(userUri, partName);
            var isCaseSensitive = userUriPart.isCaseSensitive() && contextOptions.caseSensitive;
            if (!isCaseSensitive && matchItem.flags.indexOf('i') === -1) {
                matchItem = new RegExp(matchItem.source, matchItem.flags + 'i');
            }
            var rawValue = (userUriPart.toString() || '') + '';
            if (!matchItem.test(rawValue)) {
                return null;
            }
            return new _MatchFragment2.default(rawValue, _defineProperty({}, paramName, rawValue));
        };
    } else {
        var PartConstructor = getPartConstructor(partName);
        var templateUriPart = new PartConstructor(matchItem);
        return function (userUri, contextOptions) {
            var templateUriPartString = templateUriPart.toLowerCase(!contextOptions.caseSensitive).toString();
            var userUriPart = getUserUriPart(userUri, partName);
            var userUriPartString = userUriPart.toLowerCase(!contextOptions.caseSensitive).toString();
            if (userUriPartString === templateUriPartString) {
                var rawValue = userUriPart.toString();
                return new _MatchFragment2.default(rawValue, _defineProperty({}, paramName, rawValue));
            }
            return null;
        };
    }
};

var getFallbackMatchFunction = exports.getFallbackMatchFunction = function getFallbackMatchFunction(_ref7) {
    var partName = _ref7.partName,
        paramName = _ref7.paramName,
        getUserUriPart = _ref7.getUserUriPart;

    return function (userUri, contextOptions) {
        var matchParams = {};
        var userUriPart = getUserUriPart(userUri, partName);
        matchParams[paramName] = userUriPart.toString();
        return new _MatchFragment2.default(matchParams[paramName], matchParams);
    };
};

var convertMatchItemsToFunctions = exports.convertMatchItemsToFunctions = function convertMatchItemsToFunctions(matchItems, converterOptions) {
    var matchFunctions = matchItems.map(function (matchItem) {
        return getMatchFunctionByItem(matchItem, converterOptions);
    });
    if (!matchFunctions.length) {
        matchFunctions.push(getFallbackMatchFunction(converterOptions));
    }
    return matchFunctions;
};

var getGenerateFunctionByItem = exports.getGenerateFunctionByItem = function getGenerateFunctionByItem(generateItem, _ref8) {
    var partName = _ref8.partName,
        paramName = _ref8.paramName;

    var PartConstructor = getPartConstructor(partName);
    return function (userParams, contextOptions) {
        contextOptions.paramName = paramName;
        var rawValue = typeof generateItem === 'function' ? generateItem(userParams, contextOptions) : generateItem;
        var parsedValue = new PartConstructor(rawValue);
        return parsedValue;
    };
};

var getParamGenerateFunction = exports.getParamGenerateFunction = function getParamGenerateFunction(_ref9) {
    var partName = _ref9.partName,
        paramName = _ref9.paramName;

    var PartConstructor = getPartConstructor(partName);
    return function (userParams) {
        var userParam = userParams[paramName];
        var parsedValue = new PartConstructor(userParam);
        return parsedValue;
    };
};

var getFallbackGenerateFunction = exports.getFallbackGenerateFunction = function getFallbackGenerateFunction(_ref10) {
    var partName = _ref10.partName,
        paramName = _ref10.paramName;

    return function (userParams, contextOptions) {
        var parsedValue = contextOptions.router.getDefaultPart(partName);
        return parsedValue;
    };
};

var convertGenerateItemsToFunctions = exports.convertGenerateItemsToFunctions = function convertGenerateItemsToFunctions(generateItems, converterOptions) {
    var hasFunction = generateItems.some(function (generateItem) {
        return typeof generateItem === 'function';
    });
    var generateFunctions = hasFunction ? [] : [getParamGenerateFunction(converterOptions)];
    generateFunctions = generateFunctions.concat(generateItems.map(function (generateItem) {
        return getGenerateFunctionByItem(generateItem, converterOptions);
    }));
    generateFunctions.push(getFallbackGenerateFunction(converterOptions));
    return generateFunctions;
};